/********************************************************
 * Data Visualization - 2D Matrix Demo                  *
 * Tested in Processing 3                               *     
 *                                                      *
 * Author: Han-Wei Kung                                 *
 *                                                      *
 ********************************************************/

/*
Step One: Upload CSV to Table & Get basic information of data. 
Step Two: Write data from Table to 2D Array & Have max and min value of data.
Step Three: Visualize 2D Matrix.
Step Four: Draw labels and title.
*/

Table table;
/* row = category, col = year */
int numClass, numYear;  

float[][] dataMatrix;
float maxVal, minVal;

// rects
int horMargin = 160;
int verMargin = 100;
float cellWidth, cellHeight;
float ellipseWidth, ellipseHeight;

// labels
int startYear = 2006;
int endYear = 2014;

PImage colorBar;

//Only run one time
void setup() {
  size(1280, 720); // setup the size of the window
  // In Processing 3, the size() function must be the first line of code inside setup()
  // and needs to use numbers instead of variables. 

  table = loadTable("MAT259_DataQuery.csv", "header");
  numClass = table.getRowCount();
  numYear = table.getColumnCount() - 1;
  println("Rows: " + numClass + " Columns: " + numYear);
  
  colorBar = loadImage("color_spectrum.jpg");
  
  
  maxVal = 0;
  dataMatrix = new float[numClass][numYear];
  for (int j = 0; j < numClass; j++) {
    for (int i = 0; i < numYear; i++) {
      dataMatrix[j][i] = table.getFloat(j, i + 1);
    }
  }
  
  maxVal = max2D(dataMatrix);
  minVal = min2D(dataMatrix);
}

//Refresh the canvas every frame
void draw() {  
  // println(hour() + ":" + minute() + ":" + second());
  background(255);
  
  // Calculate the width and height for each cell
  cellWidth = ((width - 2*horMargin) / numYear);
  cellHeight = (height - 2*verMargin) / numClass;
  
  // Draw 2D Matrix on canvas
  for (int i = 0; i < numYear; i ++) {
    for (int j = 0; j < numClass; j++) {
      color c = colorBar.get( (int)map(j, 0, numClass, 1, colorBar.width) - 1, colorBar.height/2 );

      ellipseWidth = map(dataMatrix[j][i], minVal, maxVal, 10, cellWidth);
      ellipseHeight = map(dataMatrix[j][i], minVal, maxVal, 5, cellHeight);
      
      fill(c);
      ellipse(horMargin + cellWidth * j + cellWidth / 2, verMargin + cellHeight * i + cellHeight / 2, ellipseWidth, ellipseHeight);
    }
  }
  
  fill(0);
  textSize(32);
  textAlign(CENTER, BOTTOM);
  text("# of books checked out in Seattle Public Library", width / 2, verMargin / 2);
  
  // Add YEAR labels
  fill(0);
  textSize(11);
  for (int i = 0; i <= endYear - startYear; i ++) {
    textAlign(RIGHT, CENTER);
    text(startYear + i, horMargin, verMargin + cellHeight / 2 + cellHeight * i);
  }
  
  drawLabels();
}